## SETS 
param NB>=0, integer ;
set BARRAS:= 1..NB ;
param NL>=0, integer ;
set LINEAS:= 1..NL ;
param NG>=0, integer ;
set GENERADORES:= 1..NG ;
param NC>=0, integer ;
set CONSUMOS:= 1..NC ;


## PARAMETERS 
param slack ;
param horas ;
param sbase ;
param cens ;
param lepspu ;
# Data lineas
param BI{i in LINEAS} ;           # barra i
param BJ{i in LINEAS} ;           # barra j
param XPU{i in LINEAS} ;          # reactancia [pu]
param RPU{i in LINEAS} ;          # resistencia [pu]
param TMAX{i in LINEAS} ;         # limite transmision [MW]
# Data generadores
param BG{i in GENERADORES} ;      # barra conexion
param PMIN{i in GENERADORES} ;    # potencia minima generable
param PMAX{i in GENERADORES} ;    # potencia maxima generable
param CVAR{i in GENERADORES} ;    # costo lineal generacion [US$/MW/h]
# Data consumos
param BC{i in CONSUMOS} ;         # barra conexion
param PDEM{i in CONSUMOS} ;       # potencia demandada
# Parametros derivados
param GPU{i in LINEAS} ;
param BPU{i in LINEAS} ;
param DELTALIM{i in LINEAS} ;
param NT{i in LINEAS} ;
param DELTAI{i in LINEAS, j in 0..2*NT[i]} ;
param BPD2{i in LINEAS, j in 1..2*NT[i]-1} ;
param SLD2{i in LINEAS, j in 1..2*NT[i]} ;


## VARIABLES 
# Angulo tension [rad]
var THETA{i in BARRAS} ;
# Angulos tension lineas [rad]
var DELTA{i in LINEAS}, >=-DELTALIM[i], <=DELTALIM[i] ;
var DELTA2{i in LINEAS}, >=0. ;
# Potencia generada [MW]
var PGEN{i in GENERADORES}, >=PMIN[i], <= PMAX[i]  ;
# Demanda no suministrada [MW]
var DNS{i in BARRAS}, >=0. ;


## OBJECTIVE 
# Costo de operacion y falla [US$] ...
minimize Costo: sum{i in GENERADORES} CVAR[i]*horas*PGEN[i] + sum{i in BARRAS} cens*horas*DNS[i] ;

                            
## CONSTRAINTS 
# Balances electricos por barra ...
subject to C1a{i in BARRAS}: 
    horas*DNS[i] + sum{j in GENERADORES: BG[j]==i} horas*PGEN[j] -
    sum{j in LINEAS: BI[j]==i} horas*sbase*(BPU[j]*(THETA[BI[j]]-THETA[BJ[j]])+0.5*GPU[j]*DELTA2[j]) -
    sum{j in LINEAS: BJ[j]==i} horas*sbase*(BPU[j]*(THETA[BJ[j]]-THETA[BI[j]])+0.5*GPU[j]*DELTA2[j]) ==
    sum{j in CONSUMOS: BC[j]==i} horas*PDEM[j] ;
subject to C1b: THETA[slack] = 0. ;
# Angulo lineas ...
subject to C2{i in LINEAS}: THETA[BI[i]]-THETA[BJ[i]] == DELTA[i] ;
# Perdidas lineas ...
subject to C3{i in LINEAS}: DELTA2[i] == << {j in 1..2*NT[i]-1} BPD2[i,j] ; {j in 1..2*NT[i]} SLD2[i,j] >> DELTA[i] ;

