/*****************************************/
/**                SETS                 **/
/*****************************************/
param NX>=0, integer;
param NY>=0, integer;
param NZ>=0, integer;
param NB>=0, integer;
set Blocks := 0..NB-1;
param NA>=0, integer;
set Arcs := 0..NA-1;
param NP>=0, integer;
set Periods := 0..NP-1;
param NW>=0, integer;
set Weights := 0..NW-1;

/*****************************************/
/**         INPUT PARAMETERS            **/
/*****************************************/
param BlockWeight{b in Blocks, w in Weights};
param BlockUtility{b in Blocks, p in Periods};
param Capacity{w in Weights, p in Periods};
param ArcTail{a in Arcs};
param ArcHead{a in Arcs};

/*****************************************/
/**              VARIABLES              **/
/*****************************************/
var x{b in Blocks, p in Periods}, binary;
var s{w in Weights, p in Periods}, >= 0;

/*****************************************/
/**             OBJECTIVE              **/
/*****************************************/
maximize Utility:  sum{b in Blocks, p in Periods} BlockUtility[b,p] * (x[b,p]-(if (p>0) then (x[b,p-1])));
                
/*****************************************/
/**            CONSTRAINTS              **/
/*****************************************/
subject to NETWORK_SPACE{a in Arcs, p in Periods}:  x[ArcTail[a],p] - x[ArcHead[a],p] >= 0 ;
subject to NETWORK_TIME{b in Blocks, p in Periods}:  x[b,p] - (if (p>0) then (x[b,p-1])) >= 0 ;
subject to WEIGHT_KNAPSACK{w in Weights, p in Periods}:  sum{b in Blocks} BlockWeight[b,w] * (x[b,p]-(if (p>0) then (x[b,p-1]))) <= Capacity[w,p] ;

