/*****************************************/
/**                SETS                 **/
/*****************************************/
param NP>0, integer;
set Periods := 1..NP;
param NG>0, integer;
set Generators := 1..NG;

/*****************************************/
/**         INPUT PARAMETERS            **/
/*****************************************/
param ENSPRICE;
param DEMAND{p in Periods};
param RESERVE{p in Periods};
param NT{g in Generators}, integer;
param NS{g in Generators}, integer;
param INIS{g in Generators}, integer;
param INIP{g in Generators};
param NE{g in Generators}, integer;
param TIME{g in Generators, n in 1..NT[g]*NS[g]}, integer;
param STATE{g in Generators, n in 1..NT[g]*NS[g]}, integer;
param ENABLE{g in Generators, n in 1..NT[g]*NS[g]}, binary;
param TAIL{g in Generators, e in 1..NE[g]}, integer;
param HEAD{g in Generators, e in 1..NE[g]}, integer;
param PMIN{g in Generators, e in 1..NE[g]};
param PMAX{g in Generators, e in 1..NE[g]};
param POWC{g in Generators, e in 1..NE[g]};
param CFIX{g in Generators, e in 1..NE[g]};
param CVAR{g in Generators, e in 1..NE[g]};
param pg{g in Generators, e in 1..NE[g]};

/*****************************************/
/**              VARIABLES              **/
/*****************************************/
var x{g in Generators, e in 1..NE[g]}, binary;
var y{g in Generators, e in 1..NE[g]}, >= 0;
var r{p in Periods}, >= 0;

/*****************************************/
/**             OBJECTIVE              **/
/*****************************************/
minimize TotalCost:  sum{g in Generators, e in 1..NE[g]} (CFIX[g,e]+CVAR[g,e]*PMIN[g,e])*x[g,e] 
								+ sum{g in Generators, e in 1..NE[g]} CVAR[g,e]*(PMAX[g,e]-PMIN[g,e])*y[g,e]
												+ sum{p in Periods} ENSPRICE * r[p] ;

/*****************************************/
/**            CONSTRAINTS              **/
/*****************************************/
subject to NETWORK_FLUX{g in Generators, n in 1..NT[g]*NS[g]: (ENABLE[g,n]>0) and (TIME[g,n]<NP) }:  
		sum{e in 1..NE[g]: (TAIL[g,e]==n) } x[g,e] - sum{e in 1..NE[g]: (HEAD[g,e]==n) } x[g,e] == (if ((STATE[g,n]==INIS[g]) && (TIME[g,n]==0)) then 1 else 0) ;
subject to YMAX{g in Generators, e in 1..NE[g]}:  
		y[g,e] <= x[g,e] ;
subject to POWER_BALANCE{p in Periods}:  
		r[p] + sum{g in Generators, e in 1..NE[g]: (TIME[g,HEAD[g,e]]==p) } (PMIN[g,e]*x[g,e]+(PMAX[g,e]-PMIN[g,e])*y[g,e]) == DEMAND[p] ;



